"""
Helper functions
"""
import os

VERSION = "0.0.7"


def get_seconds(time_str):
    """Get Seconds from time."""
    hour, minutes, seconds = time_str.split(':')
    return int(hour) * 3600 + int(minutes) * 60 + int(seconds)


def run_bash(command):
    """
    It is a helper function to run a given command
    :param command: to run by the OS
    """
    os.system(command)


def parse_time(time):
    """
    PARSING a given TIME to FFMPEG format
    """
    _temp_time_split = time.split(',')
    _new_time = _temp_time_split[0] + "."
    if "\n" in _temp_time_split[1]:
        _new_time += _temp_time_split[1][:-1]
    else:
        _new_time += _temp_time_split[1]
    return _new_time


def create_folder(folder_name):
    """
    if the folder doesnt create. it is creating
    """
    if not os.path.exists(folder_name):
        os.makedirs(folder_name)


def is_url_valid(youtube_url):
    """
    Checks a given url is valid or not
    :param youtube_url: is URL
    :return: True if the URL is valid, otherwise False
    """
    if "https://www.youtube.com/watch?v=" not in youtube_url:
        return False
    return True


def progress(percent=0, delim=100, width=30):
    """
    To show progress in the screen
    """
    left = width * percent // delim + 20
    right = width - left + 18
    print('\rCropping files: [', '#' * left, ' ' * right, ']',
          f' {percent:.0f} ', sep='', end='', flush=True)



"""
It is a test file that checks the package working.
"""
import os
import random

IS_ALL_TESS_SUCCESS = True
print("\n\033[94mChecking the installation of the package ...\033[0m")
try:
    from pypi_learn.youtube_srt_mp3 import YoutubeSrtMp3

    print("\033[0;32;49mThe module is imported SUCCESSFULLY\033[0m")
except ImportError as module_not_installed:
    print('\033[91m' + module_not_installed.name + "\033[0m module is not found")
    IS_ALL_TESS_SUCCESS = False
NUM_SUBTITLES = 193  # total number of the subtitles
TEST_SRT_FILE_PATH = "test/test_srt_file/Ot0vYkRT1Mg.srt"
DOWNLOAD_PATH = "test/test_downloads_cropped"
PATH_CROPPED_FILE = DOWNLOAD_PATH + "/Ot0vYkRT1Mg"
URLS_FILE_PATH = "test/urls.txt"


def read_test_srt_file():
    """
    to compare random subtitles
    """
    all_lines_in_srt = open(TEST_SRT_FILE_PATH, 'r').readlines()

    # To test random subtitle
    def _test_compare_random_subs():
        # generate random number to check
        num_rand = random.randint(0, NUM_SUBTITLES)
        # get subtitle from the cropped file
        sub_from_the_random_txt_file = open(PATH_CROPPED_FILE + "/Ot0vYkRT1Mg_crop_"
                                            + str(num_rand) + ".txt").readlines()[0]

        # calculate the index of the random generated sub

        real_index = num_rand * 4 - 2
        if num_rand == 0:
            real_index = 2

        # get random sub text from the real srt file
        real_random_sub = all_lines_in_srt[real_index][:-1]
        assert sub_from_the_random_txt_file == real_random_sub

    return _test_compare_random_subs()


def _download_crop_files():
    """
    it's  downloading and cropping files
    """
    return YoutubeSrtMp3(urls_file_path=URLS_FILE_PATH, save_dir=DOWNLOAD_PATH).convert()


def test_total_number_of_cropped_files():
    """
    to test number rof files
    """
    total_num_mp3 = 0
    total_num_srt = 0
    for file in os.listdir(DOWNLOAD_PATH + "/Ot0vYkRT1Mg"):
        if file.endswith('.mp3'):
            total_num_mp3 += 1
        elif file.endswith('.txt'):
            total_num_srt += 1

    return total_num_srt + total_num_mp3 == NUM_SUBTITLES * 2 + 1


def main():
    """
    it tests and prints the results.
    """
    success_count = 0
    if IS_ALL_TESS_SUCCESS is False:
        print("\033[91m Please check the module name to pass other test!\033[0m")
    else:
        success_count += 1
        # print("##########################################################################\n")
        # test download and cropped files
        print("\n\033[94mChecking to crop and download files...\033[0m")
        if _download_crop_files() is False:
            print("\033[91mSomething went wrong while downloading or cropping files. \033[0m")
        else:
            success_count += 1
            print("\033[0;32;49mDownloading and cropping are  DONE!"
                  "PASSED\033[0m")
        # print("##########################################################################\n")

        print("\n\033[94mChecking the total number of downloaded and cropped files...\033[0m")
        if test_total_number_of_cropped_files() is False:
            print("\033[91mThe total number of files does not "
                  "equal to real total files number. \033[0m")
        else:
            success_count += 1
            print("\033[0;32;49mThe total number of files is EQUAL. PASSED!\033[0m")
        # test the random subtitle between real sub and downloaded file
        # print("##########################################################################\n")

        print("\n\033[94mComparing the random text which is downloaded "
              "in the previous test with the real subtitle...\033[0m")
        if read_test_srt_file() is False:
            print("\033[91mFAILED! Two compared subtitles aren't equal.\033[0m")
        else:
            success_count += 1
            print("\033[0;32;49m Two compared subtitles are EQUAL. PASSED\033[0m")
        # print("##########################################################################\n")

    print("\n\033[94m##########################################################################\033[0m")
    print("\033[0;32;49m\t\t" + str(success_count) + " of the 4 tests were PASSED\033[0m")
    print("\033[94m##########################################################################\033[0m\n")
    if success_count < 4:
        message_error = "\033[91mFAILED! Not PASSED " + str(4 - success_count) + " tests.\033[0m"
        raise Exception(message_error)


if __name__ == '__main__':
    main()



"""
It is a file that crops the mp3 and srt file by the durations
which is getting from the srt file.
"""
import os
from ..helper.helper import run_bash, parse_time


class CropMp3Srt:
    """
    Gets the mp3 and srt files to crop
    """

    def __init__(self, filepath):
        self._path = filepath

    def crop(self):
        """
        It is a public function which is used from the other packages or files
        to perform main purpose
        """
        dirs = os.listdir(self._path)
        for file in dirs:
            if not file.startswith("crop_"):
                if file.endswith(".srt"):
                    self._crop_mp3_srt(file)

    def _crop_mp3_srt(self, filename):
        # extract video_id from the filename
        _video_id = filename[:-4]
        # read from the given file.
        with open(self._path + "/" + filename, 'r') as _filehandle:
            _filecontent_list = _filehandle.readlines()

            # it starts at index one and increases at adding 4
            for line in range(1, len(_filecontent_list), 4):
                _current_sub_num = _filecontent_list[line - 1][:-1]

                # to split the times (start and end times)
                _split__current_times = _filecontent_list[line].split(' ')

                # crop mp3 by the current times
                # PARSE TIMES
                # before given time, the duration time will get
                # as _split__current_times[:-1],
                # because it has "\n"
                _current_start = parse_time(_split__current_times[0])
                _current_end = parse_time(_split__current_times[2])
                # CALL THE CROPPING MP3 func
                self._crop_mp3(_video_id, _current_sub_num,
                               _current_start, _current_end)

                # CALL the function which WILL SAVE THE CURRENT
                # SUBTITLE to the TXT FILE
                self._save_sub_txt(_video_id, _current_sub_num,
                                   _filecontent_list[line + 1])

    def onetime_save_cropped_mp3_srt(self, video_id, sub_id,
                                     current_sub, start_time, end_time):
        """
        Saves each cropped file and performing cropping of the mp3 file
        :param video_id: is the current video id
        which is using to save cropped files
        :param sub_id: is using for naming to save cropped files
        :param current_sub: it saves to the new cropped text file
        :param start_time: is used to crop the mp3 file
        :param end_time: is used to crop the mp3 file
        """
        _current_start = parse_time(start_time)
        _current_end = parse_time(end_time)
        self._crop_mp3(video_id, str(sub_id), _current_start, _current_end)
        self._save_sub_txt(video_id, str(sub_id), current_sub)

    def _save_sub_txt(self, video_id, sub_id, current_subtitle):
        _new_srt_name = self._path + "/" + video_id \
                        + "_crop_" + sub_id + ".txt"
        with open(_new_srt_name, 'w+') as new_cropped_file:
            new_cropped_file.write(current_subtitle)
        new_cropped_file.close()

    def _crop_mp3(self, video_id, sub_id, start_time, end_time):
        """
        video_id is the video id from the youtube
        sub_id is the index from the subtitle list
        start time the starting point to crop
        """
        input_filename = self._path + "/" + video_id + ".mp3"
        new_file_name = self._path + "/" + video_id + "_crop_" + sub_id + ".mp3"

        # -n means never override
        command = "ffmpeg -n -i " + input_filename + " -ss  " + \
                  start_time + " -to " + end_time + " -c copy " \
                  + new_file_name + " 2> /dev/null"

        run_bash(command)



"""
This is a file which performs conversions
"""
import os
import datetime
from youtube_transcript_api import YouTubeTranscriptApi
import youtube_dl
from .crop_mp3_srt.crop_mp3_srt import CropMp3Srt
from .helper.helper import progress, create_folder, is_url_valid


class YoutubeSrtMp3:
    """
     This class takes a file path which contains all youtube URLs,
     and gets mp3, and creates srt files for each youtube URL.
    """

    def __init__(self, save_dir='downloads_convert', urls_file_path=""):
        self._save_dir = save_dir
        self._url_file_path = None if urls_file_path == "" else urls_file_path
        self._ydl_opts = None
        self._video_title = ""
        self._video_id = ""
        self.path = ""
        # create a folder with the given folder name
        create_folder(self._save_dir)

        # times to calculate
        self.hour_ms = 3600000
        self.min_ms = 60000
        self.sec_ms = 1000

    def convert(self):
        """
        it's a public function that performs main task
        -checking a file that contains URLs for empty
        -read all lines in the file
        -each line is a youtube URL to get mp3 and srt file
        """
        res = False
        if self._url_file_path is None:
            print("Please give a url file path.")
        else:
            _filesize = os.path.getsize(self._url_file_path)
            if _filesize == 0:
                print("The %s is empty." % self._url_file_path)
            else:

                # read line from the file and convert it
                file1 = open(self._url_file_path, 'r')
                lines = file1.readlines()
                # print("LINES DATA:\n{}".format(lines))
                for line in lines:
                    # print(line.strip()+"*")
                    youtube_url = line.strip()

                    # check youtubeurl is valid or not
                    # if it is valid, convert to mp3, srt file
                    # print info to screen and continue

                    if is_url_valid(youtube_url):
                        # self.__check_url_is_available(youtube_url)
                        print("The URL is okay.")
                        res = self.get_mp3_srt(line)
                    else:
                        print('!# %s is not valid url' % youtube_url)
                if res is True:
                    print("\033[0;32;49mDownloading and cropping are DONE!\033[0m")
                    return res
        return False

    def __set_file_details(self, video_url):
        """
        get video information
        """
        ydl_opt = self.__set_video_download_options(True)
        with youtube_dl.YoutubeDL(ydl_opt) as ydl:
            info_dict = ydl.extract_info(video_url, False)
            self._video_id = info_dict.get("id", None)

            temp_name = info_dict.get('title', None)
            if ':' in temp_name:
                split_word = temp_name.split(':')
                self._video_title = split_word[0] + ' -' + split_word[1]
            else:
                self._video_title = temp_name

    def get_mp3_srt(self, youtube_url):
        """
           convert video url to mp3 and srt file.
           1-convert  to mp3 and save video details(or extract video details)
        """
        self.__set_file_details(youtube_url)
        # convert to mp3
        self.__url_to_mp3(youtube_url)

        # convert to srt file with the video title
        self.__url_to_srt_file()
        return True

    def __url_to_srt_file(self):
        srt = YouTubeTranscriptApi.get_transcript(
            self._video_id, languages=["tr", "en"])
        # Get the length of the all subtitles
        _len_of_all_srt = len(srt)

        # Create a path of the srt file to save it
        path = self._save_dir + '/' + self._video_id
        print("PATH {}".format(path))

        # Create a folder if the path doesnt exit
        create_folder(path)
        filepath = path + "/" + self._video_id + ".srt"

        # create an object to crop mp3 and srt files
        save_each_subtitle = CropMp3Srt(path)
        with open(filepath, 'w+') as new_srt_file_write:
            add_sec = 0
            # from 0 to 373 if the len of the subtitle is 375
            for cur_idx in range(0, _len_of_all_srt):
                progress(percent=cur_idx, delim=_len_of_all_srt, )
                # print(srt[cur_idx])
                # GET current SUBTITLE
                current_subtitle = srt[cur_idx].get('text')

                # START POINT
                s_time = srt[cur_idx].get("start") + add_sec
                start_time_str = str(self.milliseconds_to_time(s_time))

                # END POINT
                # the end point of the current subtitle will be
                # the start time of the second subtitle
                # BUT the last subtitle will be the sum of duration
                # time and start time, it showed in the 'else' condition
                _dur_time_str = ""
                _line_after_subs = ""
                if cur_idx != _len_of_all_srt - 1:  # index if smaller than 374
                    dur_time = srt[cur_idx + 1].get("start")
                    # compare between the duration of the current time
                    # and the second subtitle start time
                    # get the current time duration
                    cur_sub_dur = srt[cur_idx].get("duration") + s_time
                    the_next_sub_start_time_of = srt[cur_idx + 1].get("start")
                    diff_time = the_next_sub_start_time_of - cur_sub_dur
                    add_sec = 0
                    if diff_time < 0:
                        if diff_time > -0.510:
                            add_sec = diff_time
                        else:
                            add_sec = -0.510
                        dur_time += add_sec
                    elif diff_time > 0:
                        if diff_time < 0.510:
                            add_sec = diff_time
                        else:
                            add_sec = 0.490
                        dur_time += add_sec

                    _dur_time_str = str(self.milliseconds_to_time(dur_time))
                    _line_after_subs = "\n\n"
                else:
                    dur_time = srt[cur_idx].get("duration")
                    _dur_time_str = str(self.milliseconds_to_time(dur_time
                                                                  + s_time))
                    _line_after_subs = "\n"
                new_srt_file_write.write(str(cur_idx + 1) + "\n" +
                                         start_time_str + " --> " +
                                         _dur_time_str + "\n"
                                         + current_subtitle + _line_after_subs)
                save_each_subtitle.onetime_save_cropped_mp3_srt(self._video_id,
                                                                cur_idx + 1,
                                                                current_subtitle,
                                                                start_time_str,
                                                                _dur_time_str)
            new_srt_file_write.close()
            print("\n")
            return

    def __url_to_mp3(self, youtube_url):
        self._ydl_opts = self.__set_video_download_options(is_info=False)
        self.path = self._save_dir + self._video_id
        with youtube_dl.YoutubeDL(self._ydl_opts) as ydl:
            # download video and convert to mp3
            ydl.download([youtube_url])
            # extract video information's

    def __set_video_download_options(self, is_info):
        return {
            'format': 'bestaudio/best',
            'postprocessors': [{
                'key': 'FFmpegExtractAudio',
                'preferredcodec': 'mp3',
                'preferredquality': '192',
            }],
            'outtmpl': self._save_dir + '/%(id)s.%(ext)s' if is_info is True else self._save_dir +
                                                                                  '/%(id)s/%(id)s.%(ext)s',

        }

    def milliseconds_to_time(self, millis_time):
        """
        it is a func. which convert ms to time as 00:15:15.25
        """
        millis_time = millis_time * 1000
        # print("millis_time:{}".format(millis_time))

        start_hh = int(millis_time / self.hour_ms)
        if start_hh > 0:
            current = millis_time - (start_hh * self.hour_ms)
        else:
            current = millis_time
        # print("current to min:{}".format(current))
        start_mm = int(current / self.min_ms)

        if start_mm > 0:
            current = current - (start_mm * self.min_ms)

        start_ss = int(current / self.sec_ms)

        if start_ss > 0:
            current = current - (start_ss * self.sec_ms)

        current = current % self.sec_ms

        milli_sec = current
        # print("Curr:{},{},{},{}".format(start_hh,
        # start_mm, start_ss, current))
        start_time = datetime.time(start_hh, start_mm, start_ss)
        result_time = "{},{}".format(start_time, int(milli_sec))
        return result_time



# __init__.py
"""
it is used for the setup.py
"""

import sys
import argparse
from pypi_learn.youtube_srt_mp3 import YoutubeSrtMp3
import pypi_learn.helper as package_version

# Version of the pypi_learn package

__version__ = package_version.helper.VERSION


def main():
    """
    It is a helper function which checks arguments when its package is running
    """
    parser = argparse.ArgumentParser(description='To convert the Youtube URL'
                                                 'to mp3 and srt file.')
    parser.add_argument("-sv", '--save_path', type=str,
                        help="Path to save converted files"
                             "(default: downloads_convert)")
    parser.add_argument('-ufp', '--url_file_path', type=str,
                        required=True,
                        help="A file which contains youtube URLs")

    args = parser.parse_args()

    if args.save_path is None:
        print(' # -Path of converted files set to '
              'default is \'downloads_convert\'')
        YoutubeSrtMp3(urls_file_path=args.url_file_path).convert()
    else:
        YoutubeSrtMp3(urls_file_path=args.url_file_path,
                      save_dir=args.save_path).convert()


if __name__ == '__main__':
    sys.exit(main())



"""
to publish our package
"""
import os
from setuptools import setup


def read(filename):
    """
    reading README file
    """
    with open(os.path.join(os.path.dirname(__file__), filename)) as _in:
        return _in.read()


# This call to setup() does all the work
setup(
    name="pypi_learn",
    # The version of this library.
    # Read this as
    #   - MAJOR VERSION 1
    #   - MINOR VERSION 0
    #   - MAINTENANCE VERSION 0
    version="0.0.7",
    description="Generates a dataset for the Turkish speech recognition.",
    # description and Project name| library name
    long_description=read('README.md'),
    long_description_content_type="text/markdown",

    author="ARDIC R&D",
    author_email="yavuz.erzurumlu@ardictech.com",
    url="https://github.com/IoT-Ignite/pypi_learn",
    # These are the dependencies the library needs in order to run.
    install_requires=[
        'youtube-channel-transcript-api',
        'youtube_transcript_api',
        'youtube-dl',
        'launchpadlib',
    ],

    py_modules=["pypi_learn", "pypi_learn/crop_mp3_srt/crop_mp3_srt",
                "pypi_learn/youtube_srt_mp3",
                "pypi_learn/helper/helper"],
    packages=['pypi_learn'],
    packages_dir={'': 'pypi_learn'},

    include_package_data=True,

    entry_points={
        "console_scripts": [
            "pypi_learn=pypi_learn:main",
        ]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: Apache Software License",
        "Intended Audience :: Developers"
    ],

    # Here are the keywords of my library.
    keywords='dataset, speech recognition, srt, youtube srt',
    license="MIT",
)
